import json
import os
import sys

from memory_profiler import memory_usage
from datetime import datetime

from pyattck import Attck

TEST_FILE = os.path.join(
    os.path.dirname(os.path.abspath(__file__)),
    'enterprise-attack.json'
)
os.environ.setdefault("ON_PIP", '_pip')


def get_log_file():
    version = ".".join([
        str(sys.version_info.major),
        str(sys.version_info.minor),
        str(sys.version_info.micro),
    ])
    return "py_{}{}.log".format(
        version, os.environ.get('ON_PIP')
    )


class AttckTest(Attck):
    @property
    def attck(self):
        return self._attck

    @attck.setter
    def attck(self, value):
        self._attck = json.load(open(TEST_FILE))

    def _read_file(self):
        self._attck = json.load(open(TEST_FILE))


def test(stdout):
    attack = AttckTest()
    stdout.write('## Actors ##')
    for actor in attack.actors:
        stdout.write('\n\t{}: {},{},{}'.format(
            actor.id,
            len(actor.malwares),
            len(actor.techniques),
            len(actor.tools),
        ))
    stdout.write('\n## Malwares ##')
    for malware in attack.malwares:
        stdout.write('\n\t{}: {},{}'.format(
            malware.id,
            len(malware.actors),
            len(malware.techniques),
        ))
    stdout.write('\n## Mitigations ##')
    for mitigation in attack.mitigations:
        stdout.write('\n\t{}: {}'.format(
            mitigation.id,
            len(mitigation.techniques),
        ))
    stdout.write('\n## Tactics ##')
    for tactic in attack.tactics:
        stdout.write('\n\t{}: {}'.format(
            tactic.id,
            len(tactic.techniques),
        ))
    stdout.write('\n## Techniques ##')
    for technique in attack.techniques:
        stdout.write('\n\t{}: {},{},{}'.format(
            technique.id,
            len(technique.actors),
            len(technique.mitigations),
            len(technique.tactics),
        ))
    stdout.write('\n## Tools ##')
    for tool in attack.tools:
        stdout.write('\n\t{}: {},{}'.format(
            tool.id,
            len(tool.actors),
            len(tool.techniques),
        ))

    stdout.write('\n## RESUME ##')
    stdout.write('\nActors: {}'.format(
        len(attack.actors),
    ))
    stdout.write('\nMalwares: {}'.format(
        len(attack.malwares),
    ))
    stdout.write('\nMitigations: {}'.format(
        len(attack.mitigations),
    ))
    stdout.write('\nTactics: {}'.format(
        len(attack.tactics),
    ))
    stdout.write('\nTechniques: {}'.format(
        len(attack.techniques),
    ))
    stdout.write('\nTools: {}\n'.format(
        len(attack.tools),
    ))
    stdout.flush()


with open(get_log_file(), 'w') as f:
    start_at = datetime.now()
    ret = memory_usage((test, (sys.stdout,)))
    mem_time_required = datetime.now() - start_at

    f.write('\n\nMemory:\n\tMax: {}Mb\n\tMin: {}Mb\n\tMedian({}): {}Mb'.format(
        max(ret),
        min(ret),
        len(ret),
        sum(ret)/len(ret)
    ))
    f.write('\nTime mem: ')
    f.write(str(mem_time_required))
